<?php

function translate($text, $target_lang) {
    $url = 'https://libretranslate.de/translate';
    $data = array(
        'q' => $text,
        'source' => 'de',
        'target' => $target_lang,
        'format' => 'text'
    );

    $options = array(
        'http' => array(
            'header'  => "Content-type: application/x-www-form-urlencoded\r\n",
            'method'  => 'POST',
            'content' => http_build_query($data)
        )
    );

    $context  = stream_context_create($options);
    $result = file_get_contents($url, false, $context);

    if ($result === FALSE) {
        return $text;
    }

    $response = json_decode($result, true);
    return $response['translatedText'] ?? $text;
}

$original_content = file_get_contents('de-DE.json');
$original_data = json_decode($original_content, true);

$language_codes = json_decode(file_get_contents('languagecodes.json'), true);

// Lade die Sprachnamen
$language_names = json_decode(file_get_contents('language_names.json'), true);

$files_created = 0;
$all_files_exist = true;

for ($i = 0; $i < count($language_codes); $i++) {
    $lang_code = $language_codes[$i];
    $file_name = $lang_code . '.json';
    
    if (file_exists($file_name)) {
        echo "Datei {$file_name} existiert bereits. Überspringe...\n";
        continue;
    }
    
    $all_files_exist = false;
    
    $translated_data = array();

    // Bestimme die Zielsprache für die Übersetzung
    $target_lang = $language_names[$lang_code] ?? 'Unknown';
    
    // Wenn die Zielsprache unbekannt ist, verwende den Basissprachcode
    if ($target_lang === 'Unknown') {
        $base_code = explode('-', $lang_code)[0];
        $target_lang = $language_names[$base_code] ?? 'Unknown';
    }

    foreach ($original_data as $key => $value) {
        if ($key === 'language') {
            $translated_data[$key] = $lang_code;
        } elseif (strpos($key, 'icon') !== false) {
            $translated_data[$key] = $value;
        } else {
            $translated_data[$key] = translate($value, $target_lang);
        }
    }

    $json_content = json_encode($translated_data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
    file_put_contents($file_name, $json_content);

    echo "Datei {$file_name} wurde erstellt.\n";
    $files_created++;

    if ($files_created >= 5) {
        break;
    }
}

if ($all_files_exist) {
    echo "Für jeden Ländercode existiert bereits eine Datei.\n";
} elseif ($files_created > 0) {
    echo "Übersetzung und Erstellung der Dateien abgeschlossen. {$files_created} neue Dateien wurden erstellt.\n";
    echo "<script>window.location.reload();</script>";
} else {
    echo "Keine neuen Dateien wurden erstellt.\n";
}