<?php
// Set the update interval (24 hours in seconds)
$updateinterval = 24 * 60 * 60;

function handleNewDevice() {
    // Check if 'did' and 'didfull' are set in POST parameters
    if (isset($_POST['did']) && isset($_POST['didfull'])) {
        $did = substr($_POST['did'], 0, 16);
        $didfull = $_POST['didfull'];
        
        // Directory to store device keys
        $deviceKeysDir = './devicekeys/';
        $filePath = $deviceKeysDir . $did . '.txt';
        
        // Check if the 'devicekeys' directory exists, if not create it
        if (!is_dir($deviceKeysDir)) {
            mkdir($deviceKeysDir, 0777, true);
        }

        // Check if the file already exists to avoid overwriting
        if (file_exists($filePath)) {
            echo "ISSET:::Error: The file already exists.";
            return;
        }

        // Check the size of 'didfull' content
        if (strlen($didfull) <= 2048) {
            // Create and write to the device key file if it does not exist
            file_put_contents($filePath, $didfull);
            echo "ISSET";
        } else {
            echo "Error: The content of 'didfull' exceeds 2KB.";
        }
    }
}

// Function to handle 'upload' action
function handleUpload() {
    // Check if all required POST parameters are set
    if (isset($_POST['did'], $_POST['aid'], $_POST['didhash'], $_POST['time'], $_POST['koto'])) {
        $did = $_POST['did'];
        $aid = $_POST['aid'];
        $didhash = $_POST['didhash'];
        $time = $_POST['time'];
        $koto = $_POST['koto'];
        $WYAdeviceID = $_POST['WYAdeviceID'];

        // Directory and file paths
        $deviceKeysDir = './devicekeys/';
        $devicesDir = './_devices/';
        $deviceKeyFile = $deviceKeysDir . $did . '.txt';

        // Check if the device key file exists and validate its contents
        if (file_exists($deviceKeyFile)) {
            $deviceKeyContent = file_get_contents($deviceKeyFile);

            // Validate the hash and time
            //echo " - ".$time ;
            //echo " - ".$WYAdeviceID ;
            //echo " - ".$didhash ;
            //echo " - ".substr(substr($deviceKeyContent, 14), 0, 32) ;

            //echo " - ".base64_encode(hash_hmac('sha256', substr(substr($deviceKeyContent, 14), 0, 32), $time, true));
            //if (hash('sha256', substr(substr($deviceKeyContent, 14), 0, 32).$time) === $didhash) {
            if (substr(substr($deviceKeyContent, 14), 0, 32) === $didhash) {
                $deviceDir = $devicesDir . $did;

                // Check if the device directory exists, if not create it
                if (!is_dir($deviceDir)) {
                    mkdir($deviceDir, 0777, true);
                }

                // Check the size of 'koto' content
                if (strlen($koto) <= 500 * 1024) {
                    // Save the 'aid.koto' file in the device directory
                    file_put_contents("./".$deviceDir . '/' . $aid . '.koto', $koto);
                    echo "OK_".$did."_".$aid;
                    $scheme = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') ? 'https' : 'http';
                    //echo $scheme."://".$_SERVER["SERVER_NAME"].dirname($_SERVER["PHP_SELF"])."/_devices/".$did."/".$aid.".koto";
                } else {
                    echo "Error: The content of 'koto' exceeds 500KB.";
                }
            } else {
                echo "Error: Hash or time validation failed.";
            }
        } else {
            echo "Error: Device key file not found.";
        }
    }
}

// Function to handle 'index' action
function handleIndex() {
    $lastIndexFile = 'lastindex.time';

    // Get the current timestamp
    $currentTimestamp = time();
    $shouldUpdateIndex = true;

    // Check if 'lastindex.time' exists and is older than the update interval
    if (file_exists($lastIndexFile)) {
        $lastIndexTime = file_get_contents($lastIndexFile);
        if ($currentTimestamp - $lastIndexTime < 24 * 60 * 60) {
            $shouldUpdateIndex = false;
        }
    }

    // Update the index if needed
    if ($shouldUpdateIndex) {
        file_put_contents($lastIndexFile, $currentTimestamp);

        $deviceKeysDir = './devicekeys/';
        $indexData = [];

        // Scan through the device keys directory
        if (is_dir($deviceKeysDir)) {
            $devices = scandir($deviceKeysDir);
            foreach ($devices as $device) {
                if ($device !== '.' && $device !== '..' && pathinfo($device, PATHINFO_EXTENSION) === 'txt') {
                    $deviceDir = './_devices/' . pathinfo($device, PATHINFO_FILENAME);
                    if (is_dir($deviceDir)) {
                        $files = scandir($deviceDir);
                        foreach ($files as $file) {
                            if ($file !== '.' && $file !== '..' && pathinfo($file, PATHINFO_EXTENSION) === 'koto') {
                                $filePath = $deviceDir . '/' . $file;
                                $content = file_get_contents($filePath);
                                $jsonContent = json_decode($content, true);
                                if ($jsonContent && isset($jsonContent['type']) && $jsonContent['type'] === 'part') {
                                    $indexData[] = [
                                        'path' => $filePath,
                                        'name' => $jsonContent['name']
                                    ];
                                }
                            }
                        }
                    }
                }
            }
        }

        // Save the index data to 'didindex.json'
        file_put_contents('./didindex.json', json_encode($indexData));
    }
}

// Main script to handle different POST actions
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_GET['action'])) {
        $action = $_GET['action'];
        switch ($action) {
            case 'newdevice':
                handleNewDevice();
                break;
            case 'upload':
                handleUpload();
                break;
            default:
                echo "Error: Unknown action.";
        }
    } else {
        echo "Error: No action specified.";
    }
} elseif ($_SERVER['REQUEST_METHOD'] === 'GET' && $_GET['action']=='index') {
    handleIndex();
} else {
    echo "Error: Unsupported request method.";
}
?>
